'============================================================================================
' File name:        validateChiNumber.vbs
' Created:          26-Jul-2002
' Author:           Fraser Moore
' Description:      Provides a function to validate CHI numbers.
' Dependencies:     
' Issues:           
'============================================================================================

Option Explicit

Const CHI_ERROR = 1
Const CHI_WARNING = 2

Const FEMALE = 0
Const MALE = 1

Const CHI_NUMBER_LENGTH = 10    ' N.B. This constraint could change in the longer term.

'=============================================================================
' Function IsValidChiNumber
' Description:  Checks if the CHI number is valid (i.e. well-formed) and
'               matches patient data.
'               N.B. This function only validates the CHI number - it cannot
'               verify that it is correct.
' Pass:         [in]  chiNumber      - patient's CHI number
'               [in]  dateOfBirth    - patient's date of birth
'               [in]  sex            - patient's sex
'				[in]  validationType - 
'               [out] severity       - CHI_WARNING or CHI_ERROR
'               [out] message        - error/warning message if invalid
' Return:       True if the CHI number is valid
' Raise Errors: None
'---------------------------------------------
' Date          Developer           Comments
' 26-Jul-2002   Fraser Moore        
'---------------------------------------------
Function isValidChiNumber(chiNumber, dateOfBirth, sex, validationType, severity, message)
	Dim blnValid
	Dim blnDobValid
	Dim blnSexValid
	Dim strMsgDob
	Dim strMsgSex
	blnValid = validateCHIFormat(chiNumber, severity, message)
'	stop
	If blnValid Then

		Select Case validationType
			Case CHI_DOB
				blnValid = validateCHIAgainstDOB(chiNumber, dateOfBirth, message)
			Case CHI_SEX
				blnValid = validateCHIAgainstSex(chiNumber, sex, message)
			Case CHI_DOB_SEX					
				blnDobValid = validateCHIAgainstDOB(chiNumber, dateOfBirth, strMsgDob)
				blnSexValid = validateCHIAgainstSex(chiNumber, sex, strMsgSex)
				blnValid = (blnDobValid And blnSexValid)
				
				If Not blnValid Then					
					If ((Not blnDobValid) And (Not blnSexValid)) Then
						message = strMsgDob & ", " & vbCrLf & strMsgSex 
					ElseIf (Not blnDobValid) Then
						message = strMsgDob
					ElseIf (Not blnSexValid) Then
						message = strMsgSex
					End If			
				End If
		End Select
		severity = CHI_WARNING
	End If
		
	isValidChiNumber = blnValid
End Function

Function validateCHIFormat(chiNumber, severity, message)
	Dim explicitDate
	Dim total, factor, index
	Dim checkDigit
	Dim chiDobDay, chiDobMonth, chiDobYear
	
	' Check the entered CHI number is numeric
	If Not IsNumeric(chiNumber) Then
		severity = CHI_ERROR
		message = "CHI number must be numeric"
		validateCHIFormat = False : Exit Function
	End If

	' Check the entered CHI number is the correct length
	If Len(chiNumber) <> CHI_NUMBER_LENGTH Then
		severity = CHI_ERROR
		message = "CHI number must be 10 digits long"
		validateCHIFormat = False : Exit Function
	End If	

	' Verify the check digit

	' Calculate check digit based on first 9 digits
	total = 0
	factor = 1
	For index = CHI_NUMBER_LENGTH-1 To 1 Step -1
		factor = factor + 1
		total = total + Mid(chiNumber, index, 1) * factor
	Next

	checkDigit = 11 * (total\11 + 1) - total

	If checkDigit = 11 Then checkDigit = 0

	If checkDigit = 10 Then
		severity = CHI_ERROR
		message = "The CHI number is invalid"
		validateCHIFormat = False : Exit Function
	End If
	
	If checkDigit <> CInt(Mid(chiNumber, 10, 1)) Then
		severity = CHI_ERROR
		' Check digit incorrect
		message = chiNumber & " is not a valid CHI number"
		validateCHIFormat = False : Exit Function
	End If

	' Check the first 6 digits of the CHI number are a valid date of birth,
	' and cross-check date of birth given	
	chiDobDay = CInt(Left(chiNumber, 2))
	chiDobMonth = CInt(Mid(chiNumber, 3, 2))
	chiDobYear = CInt(Mid(chiNumber, 5, 2))

	' Check that month is valid
	If chiDobMonth<1 or chiDobMonth>12 Then
		severity = CHI_ERROR
		message = "CHI number indicates invalid month in date of birth"
		validateCHIFormat = False : Exit Function
	End If
	
	' IsDate recognises both UK and US format dates, so...	
	explicitDate = chiDobDay & "-" & MonthName(chiDobMonth) & "-" & chiDobYear

	If Not IsDate(explicitDate) Then
		severity = CHI_ERROR
		message = "CHI number indicates invalid date of birth"
		validateCHIFormat = False : Exit Function
	End If
	
	validateCHIFormat = True
End Function

Function validateCHIAgainstDOB(chiNumber, dateOfBirth, message)	
  	Dim dobDay, dobMonth, dobYear
	Dim chiDobDay, chiDobMonth, chiDobYear
	
	If Not IsDate(dateOfBirth) Then
	    message = "The patients date of birth is not a valid date"
	    validateCHIAgainstDOB = False
	    Exit Function
	End If
	
	chiDobDay = CInt(Left(chiNumber, 2))
	chiDobMonth = CInt(Mid(chiNumber, 3, 2))
	chiDobYear = CInt(Mid(chiNumber, 5, 2))
  	
  	dobDay = Day(dateOfBirth)
  	dobMonth = Month(dateOfBirth)
  	dobYear = Year(dateOfBirth) Mod 100   ' N.B. 2-digit year

	If chiDobDay<>dobDay Or chiDobMonth<>dobMonth Or chiDobYear<>dobYear Then
		message = "CHI number does not match patient's date of birth"
		validateCHIAgainstDOB = False : Exit Function
	End If
	
	validateCHIAgainstDOB = True
End Function

Function validateCHIAgainstSex(chiNumber, sex, message)
	Dim chiSex
	If Mid(chiNumber, 9, 1) Mod 2 = 1 Then chiSex = MALE Else chiSex = FEMALE

	' Check expected sex with sex indicated by CHI number
	If sex <> chiSex Then
		message = "Sex of patient and CHI number do not agree"
		validateCHIAgainstSex = False : Exit Function
	End If
	
	validateCHIAgainstSex = True
End Function