﻿namespace SCI.Gateway.Web.Message
{

    #region Using directives

    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using System.Xml;
    using SCI.Gateway.Utility;
    using SCI.Gateway.InterfaceTypes;
    using System.Text;
    using System.Xml.Linq;


    #endregion

    /// <summary>
    /// A page for displaying validation results of pre-population XML from 3rd party systems.
    /// </summary>
    public partial class GpXmlValidation : GatewayPage
    {

        #region Public Properties

        /// <summary>
        /// Override for page title.
        /// </summary>
        public override string PageTitle
        {
            get { return "GP XML Data Validation"; }
        }



       #endregion


        #region overridden methods


        protected override void Page_Load(object sender, EventArgs e)
        {
            base.Page_Load(sender, e);
            AddClientScriptReferences();
            RetrievePostValues();
        }


        protected override void OnInit(EventArgs e)
        {
            InitializeComponent();
            base.OnInit(e);
        }



        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            this.SetPageDisplay();
        }


        #endregion

        
        #region Private methods

        private void SetPageDisplay()
        {
            List<GpXmlSchemaValidationError> errors = ValidateThirdPartyPatientXml(prePopXml.Value);
            DisplayErrors(errors);
        }

        private void InitializeComponent()
        {
            this.Load += new System.EventHandler(this.Page_Load);
        }


        /// <summary>
        /// Validates the prepopulation xml from 3rd party system.
        /// </summary>
        /// <param name="thirdPartyPatientXml">The xml to validate</param>
        /// <returns>A list of validation errors</returns>
        private List<GpXmlSchemaValidationError> ValidateThirdPartyPatientXml(string thirdPartyPatientXml)
        {
            XmlDocument thirdPartyPatient = new XmlDocument();
            thirdPartyPatient.LoadXml(thirdPartyPatientXml);
            XmlNode rootNode = thirdPartyPatient.DocumentElement;
            string schemaVersion = rootNode.Attributes.GetNamedItem("SchemaVersion").Value;
            List<GpXmlSchemaValidationError> errors = new List<GpXmlSchemaValidationError>();

            string schemaUrl =
                Request.Url.GetComponents(UriComponents.SchemeAndServer, UriFormat.UriEscaped)
                + Page.ResolveUrl(
                    "~/"
                    + ApplicationConfig.GetConfigSetting(GatewayConfigSettings.ThirdPartySchemaUrl)
                    + "-"
                    + schemaVersion
                    + ".xsd");

            try
            {
                SchemaValidation schemaValidation = new SchemaValidation(SessionContext);
                errors = schemaValidation.ValidateAgainstSchema(
                    thirdPartyPatientXml,
                    "http://www.show.scot.nhs.uk/isd/ThirdPartyInterface",
                    schemaUrl,
                    SessionContext.InstanceName);

            }
            catch (XmlException exception)
            {
                SessionContext.EventLogger.LogEvent(EventLoggerSeverity.Error, exception);
             }
            return errors;
        }



        /// <summary>
        /// Retrieves values posted to the page.
        /// </summary>
        private void RetrievePostValues()
        {
            prePopXml.Value = Server.HtmlDecode(Request["prePopXml"].ToString());
        }



        /// <summary>
        /// Displays the errors as HTML
        /// </summary>
        /// <param name="validationErrors">The errors to display</param>
        private void DisplayErrors(List<GpXmlSchemaValidationError> validationErrors)
        {
            foreach (GpXmlSchemaValidationError error in validationErrors)
            {
                Panel errorPanel = new Panel();
                Table errorTable = CreateTable(error);
                errorTable.CssClass = "validationTable";

                errorPanel.Controls.Add(errorTable);
                ErrorListPlaceholder.Controls.Add(errorPanel);
            }
            ErrorListPlaceholder.Visible = true;
        }



        

        /// <summary>
        /// Adds client side script to the page.
        /// </summary>
        private void AddClientScriptReferences()
        {
            StringBuilder jScriptFiles = new StringBuilder();
            jScriptFiles.Append("jquery-1.7.js;");
            jScriptFiles.Append("GpXmlValidation.js");
            
            SetJavaScriptFiles(jScriptFiles.ToString(), "GpXmlValidationJScripts");
        }



        private IList<TableRow> CreateTableRows(GpXmlSchemaValidationError error)
        {
            IList<TableRow> rows = new List<TableRow>();
            
            //User friendly error.
            TableRow userFriendlyError = CreateTableRow(error.UserFriendlyErrorMessage);
            userFriendlyError.Cells[0].CssClass = "userFriendlyError";
            rows.Add(userFriendlyError);

            //Detailed error.
            TableRow detailedRow = CreateTableRow(error.ErrorMessage);
            rows.Add(detailedRow);

            //The Element name.
            TableRow nameRow = CreateTableRow(error.ElementName);
            rows.Add(nameRow);

            //The element value.
            TableRow valueRow = CreateTableRow(error.ElementValue);
            rows.Add(valueRow);

            return rows;
        }


        private TableRow CreateTableRow(string text)
        {
            TableRow row = new TableRow();
            TableCell cell = new TableCell();
            cell.Text = text;
            cell.CssClass = "errorDetails";
            row.Cells.Add(cell);

            return row;
        }

        private Table CreateTable(GpXmlSchemaValidationError error)
        {
            Table errorTable = new Table();
            errorTable.CssClass = "validationTable";
            //foreach (GpXmlSchemaValidationError error in errors)
            //{
                IList<TableRow> rows = CreateTableRows(error);

                foreach (TableRow row in rows)
                {
                    errorTable.Rows.Add(row);
                }
            //}

            return errorTable;
        }

        #endregion

        
    }
}